import numpy as np
import pandas as pd
from ovretl.loads_utils.calculate_entity_loads import calculate_shipments_propositions_loads
from pandas.util.testing import assert_frame_equal


def test_calculate_entity_loads():
    loads_df = pd.DataFrame(
        data={
            "unit_weight": [10, 0, 100, 10, 10],
            "unit_length": [120, 0, 120, 120, 120],
            "unit_height": [100, 0, 100, 100, 100],
            "unit_width": [80, 0, 80, 80, 80],
            "unit_number": [1, 0, 1, 1, 1],
            "total_weight": [200, 0, np.nan, 200, 200],
            "total_volume": [0.9, 0, 0.9, 1, np.nan],
            "proposition_id": ["0", "0", "1", np.nan, np.nan],
            "shipment_id": [np.nan, np.nan, np.nan, "2", "3"],
            "hazardous": [True, True, True, False, False],
            "non_stackable": [False, True, False, False, False],
        }
    )
    result_should_be = pd.DataFrame(
        data={
            "total_weight": [200, 100, 200, 200],
            "total_volume": [0.9, 0.9, 1, 0.96],
            "total_number": [1, 1, 1, 1],
            "proposition_id": ["0", "1", np.nan, np.nan],
            "shipment_id": [np.nan, np.nan, "2", "3"],
            "taxable_weight": [200, 150, 200, 200],
            "weight_measurable": [1, 1, 1, 1],
            "hazardous": [True, True, False, False],
            "non_stackable": [True, False, False, False],
        }
    )
    result = calculate_shipments_propositions_loads(loads_df).reset_index(drop=True)
    assert_frame_equal(result, result_should_be, check_dtype=False, check_like=True)



import pandas as pd


def sum_multiple_billings_prices(shipments_with_prices: pd.DataFrame):
    return (
        shipments_with_prices.groupby(["shipment_id", "category"])
        .agg(
            {
                "price_in_eur": "sum",
                "vat_price_in_eur": "sum",
                "margin_price_in_eur": "sum",
                "purchase_price_in_eur": "sum",
                "initial_price_in_eur": "sum",
                "initial_margin_price_in_eur": "sum",
                "prices_origin": "first",
                "foresea_name": "first",
            }
        )
        .reset_index()
    )



import pandas as pd
import pytest

from ovretl.performances_utils.calculate_shipments_treatment_steps_time import calculate_shipments_treatment_steps_time


@pytest.mark.skip(reason="Need to disable timezone")
def test_calculate_shipments_treatment_steps_time(snapshot):
    events_shipment_df = pd.read_csv("./ovretl/performances_utils/tests/events_shipment.csv")
    activities_clean_df = pd.read_csv("./ovretl/performances_utils/tests/activities_clean.csv")
    shipments_df = pd.read_csv("./ovretl/performances_utils/tests/shipments_df.csv")
    result = calculate_shipments_treatment_steps_time(
        events_shipment_df=events_shipment_df, activities_clean_df=activities_clean_df, shipments_df=shipments_df
    )
    snapshot.assert_match(result)



import pandas as pd


def find_shipment_employee_name(employees_associated_df: pd.DataFrame, role: str):
    if len(employees_associated_df) == 0:
        return None
    mask_employees = employees_associated_df["role"] == role
    employees_associated_df_filtered = employees_associated_df[mask_employees].reset_index(drop=True)
    if len(employees_associated_df_filtered) > 0:
        return employees_associated_df_filtered.loc[0, "name"]
    return None


def add_employees_to_shipments(shipments_df: pd.Series, employees_df: pd.DataFrame):
    shipments_df = (
        pd.merge(
            left=shipments_df, right=employees_df[["id", "name"]], left_on="sales_owner_id", right_on="id", how="left",
        )
        .drop(["id", "sales_owner_id"], axis=1)
        .rename(columns={"name": "sales"})
    )
    shipments_df = (
        pd.merge(
            left=shipments_df,
            right=employees_df[["id", "name"]],
            left_on="operations_owner_id",
            right_on="id",
            how="left",
        )
        .drop(["id", "operations_owner_id"], axis=1)
        .rename(columns={"name": "operations"})
    )
    shipments_df = (
        pd.merge(
            left=shipments_df,
            right=employees_df[["id", "name"]],
            left_on="pricing_owner_id",
            right_on="id",
            how="left",
        )
        .drop(["id", "pricing_owner_id"], axis=1)
        .rename(columns={"name": "pricing"})
    )
    return shipments_df



import numpy as np
import pandas as pd
import datetime
from typing import Optional


START_TIME = datetime.time(8, 30)
END_TIME = datetime.time(19, 30)
TOTAL_WORKED_HOURS = END_TIME.hour - START_TIME.hour


def calculate_time_difference_within_day_in_hours(date_1: datetime.time, date_2: datetime.time):
    date_1 = datetime.datetime.combine(datetime.date.today(), date_1)
    date_2 = datetime.datetime.combine(datetime.date.today(), date_2)
    datetime_difference = date_2 - date_1
    return round(datetime_difference.total_seconds() / 3600, 2)


def calculate_business_hours_delta(date_1: Optional[datetime.datetime], date_2: Optional[datetime.datetime]):
    """
    Calculate time difference in worked hours, based on Mo-Fr, working from 8:30 to 19:30
    :param date_1:
    :param date_2:
    :return:
    """
    if date_1 is None or date_2 is None or pd.isna(date_1) or pd.isna(date_2):
        return None
    business_days_diff = np.busday_count(date_1.strftime("%Y-%m-%d"), date_2.strftime("%Y-%m-%d"))
    if business_days_diff == 0:
        return calculate_time_difference_within_day_in_hours(date_1.time(), date_2.time())
    first_day_remaining_h = max(0.0, calculate_time_difference_within_day_in_hours(date_1.time(), END_TIME))
    second_day_remaining_h = max(0.0, calculate_time_difference_within_day_in_hours(START_TIME, date_2.time()))

    return (business_days_diff - 1) * TOTAL_WORKED_HOURS + first_day_remaining_h + second_day_remaining_h



import pandas as pd


def is_import(shipment: pd.Series):
    if shipment["delivery_harbor_country"] in ["FR", "BE", "NL", "ES", "IT"] and not shipment[
        "pickup_harbor_country"
    ] in ["FR", "BE", "NL", "ES", "IT"]:
        return True
    return False


def is_abroad(shipment_is_import: bool, carriage_type: str):
    if shipment_is_import:
        if carriage_type == "precarriage":
            return True
        return False
    if carriage_type == "postcarriage":
        return True
    return False



import pandas as pd
from ovretl.transit_times.preprocess_transit_times import preprocess_transit_times
from pandas.util.testing import assert_frame_equal


def test_preprocess_transit_times():
    raw_df = pd.read_csv("./ovretl/transit_times/tests/transit_times_data.csv")
    result = preprocess_transit_times(raw_df, 20)
    result_should_be = pd.read_csv("./ovretl/transit_times/tests/preprocessed_transit_times_data.csv")
    assert_frame_equal(result.reset_index(drop=True), result_should_be.reset_index(drop=True))



import pandas as pd


def calculate_turnover(prices_df: pd.DataFrame):
    categories_to_sum = [
        "departure_truck_freight",
        "departure_fees",
        "freight",
        "arrival_fees",
        "arrival_truck_freight",
        "insurance",
    ]
    prices_df_filtered = prices_df[prices_df["category"].isin(categories_to_sum)]

    prices_df_filtered.loc[:, "turnover"] = prices_df_filtered.groupby("shipment_id")["price_in_eur"].transform(sum)
    return prices_df_filtered.drop_duplicates("shipment_id")
